// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/background

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome */

'use strict';

const browserWrapper = require('./chrome-wrapper.es6');

let dev = false;

function returnResponse(xhr, returnType) {
  if (returnType === 'json' && xhr && xhr.responseText) {
    let res;

    try {
      res = JSON.parse(xhr.responseText);
    } catch (e) {
      /* eslint-disable-next-line no-console */
      console.warn(`could not parse JSON response: ${xhr.responseText}`);
    }

    return res;
  } if (returnType === 'xml') {
    return xhr.responseXML;
  }
  return xhr.responseText;
}

/*
 * Params:
 *  - url: request URL
 *  - source: requests are internal by default. set source to 'external' for non-extension URLs
 *  - etag: set an if-none-match header
 */
function loadExtensionFile(params) {
  const xhr = new XMLHttpRequest();
  let theURL = params.url;

  if (params.source === 'external') {
    if (dev) {
      if (theURL.indexOf('?') > -1) {
        theURL += '&';
      } else {
        theURL += '?';
      }

      theURL += 'test=1';
    }

    xhr.open('GET', theURL);

    if (params.etag) {
      xhr.setRequestHeader('If-None-Match', params.etag);
    }
  } else {
    // set type xhr type tag. Safari internal xhr requests
    // don't set a 200 status so we'll check this type
    xhr.type = 'internal';
    xhr.open('GET', browserWrapper.getExtensionURL(theURL));
  }

  xhr.timeout = params.timeout || 30000;

  xhr.send(null);

  return new Promise((resolve, reject) => {
    xhr.ontimeout = () => {
      reject(new Error(`${theURL} timed out`));
    };
    xhr.onreadystatechange = () => {
      const done = XMLHttpRequest.DONE ? XMLHttpRequest.DONE : 4;
      if (xhr.readyState === done) {
        if (xhr.status === 200 || (xhr.type && xhr.type === 'internal')) {
          xhr.data = returnResponse(xhr, params.returnType);
          if (!xhr.data) {
            reject(new Error(`${theURL} returned no data`));
          }
          resolve(xhr);
        } else if (xhr.status === 304) {
          log(`${theURL} returned 304, resource not changed`);
          resolve(xhr);
        } else {
          reject(new Error(`${theURL} returned ${xhr.status}`));
        }
      }
    };
  });
}

function setDevMode() {
  dev = true;
}

function JSONfromLocalFile(path) {
  return loadExtensionFile({ url: path, returnType: 'json' });
}

function JSONfromExternalFile(theURL) {
  return loadExtensionFile({ url: theURL, returnType: 'json', source: 'external' });
}

function url(theURL) {
  return loadExtensionFile({ url: theURL, source: 'external' });
}

module.exports = {
  loadExtensionFile,
  JSONfromLocalFile,
  JSONfromExternalFile,
  url,
  setDevMode,
};
